/* neighboring8Direct.h
 */
#ifndef EFFECT_UTIL_NEIGHBORING8DIRECT_H
#define EFFECT_UTIL_NEIGHBORING8DIRECT_H

#include "osl/state/numEffectState.h"
#include "osl/direction.h"

namespace osl
{
  namespace effect_util
  {
    /**
     * 8˵ؤľܤȽꤹ
     */
    class Neighboring8Direct
    {
      class Table
      {
	struct Entry
	{
	  bool has_unblockable_effect;
	  Offset nearest;
	  Entry() : has_unblockable_effect(false), nearest(Offset::ZERO())
	  {
	  }
	};
	CArray2d<Entry,PTYPEO_SIZE,Offset32::SIZE> table;
	void init(Player);
      public:
	Table();
	bool hasEffect(const NumEffectState& state,
		       PtypeO ptypeo, Position from, 
		       Position target) const
	{
	  const Offset32 offset32 = Offset32(target, from);
	  const Entry& e = table[ptypeOIndex(ptypeo)][offset32.index()];
	  if (e.has_unblockable_effect)
	    return true;
	  if (e.nearest.zero())
	    return false;
	  assert(Ptype_Table.hasLongMove(getPtype(ptypeo)));
	  const Position nearest = from+e.nearest;
	  if (nearest.isEdge())
	  {
	    return false;
	  }
	  return state.isEmptyBetween(from, nearest, false);
	}
	Position findNearest(const NumEffectState& state,
			     PtypeO ptypeo, Position from, 
			     Position target) const
	{
	  const Offset32 offset32 = Offset32(target, from);
	  const Entry& e = table[ptypeOIndex(ptypeo)][offset32.index()];
	  if (e.has_unblockable_effect)
	    return from;
	  if (e.nearest.zero())
	    return Position::STAND();
	  assert(Ptype_Table.hasLongMove(getPtype(ptypeo)));
	  const Position nearest = from+e.nearest;
	  if (!nearest.isEdge() && state.isEmptyBetween(from, nearest, false))
	    return nearest;
	  return Position::STAND();
	}
      };
      static const Table table;
    public:
      /**
       * ptypeo ζfromtarget8˵ľܤĤ
       */
      static bool hasEffect(const NumEffectState& state,
			    PtypeO ptypeo, Position from, 
			    Position target)
      {
	return table.hasEffect(state, ptypeo, from, target);
      }
      static Position findNearest(const NumEffectState& state,
				  PtypeO ptypeo, Position from, 
				  Position target)
      {
	return table.findNearest(state, ptypeo, from, target);
      }
    private:
      static bool hasEffectFromTo(const NumEffectState& state,
				  PtypeO ptypeo, Position from, 
				  Position target, Direction d);
    public:
      static bool hasEffectNaive(const NumEffectState& state,
				 PtypeO ptypeo, Position from, 
				 Position target);
    };

  } // namespace effect_util
  using effect_util::Neighboring8Direct;
} // namespace osl

#endif /* EFFECT_UTIL_NEIGHBORING8DIRECT_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
