/* fromEffect.h
 */
#ifndef OSL_EVAL_FROMEFFECT_H
#define OSL_EVAL_FROMEFFECT_H

#include "osl/eval/experimental/weights.h"
#include "osl/eval/evalTraits.h"
#include "osl/state/numEffectState.h"

namespace osl
{
  namespace eval
  {
    namespace experimental
    {
      class FromEffect
      {
      public:
	static std::pair<int,int> count(Player me, const NumEffectState& state, Position position);
	enum { DIM = 9*PTYPE_SIZE };

	static int eval(const NumEffectState& state, const Weights& values);
	static int evalWithUpdate(const NumEffectState& state, Move moved, int last_value, const Weights& values);
	template <Player P>
	static int evalWithUpdate(const NumEffectState& state, Move moved, int last_value, const Weights& values);

	static int pieceValue(const NumEffectState& state, Piece p, const Weights& values);
	static int index(Piece p, const NumEffectState& state)
	{
	  if (p.owner() == BLACK)
	    return index<BLACK>(p, state);
	  else
	    return index<WHITE>(p, state);
	}
	template <Player P>
	static int index(Piece p, const NumEffectState& state)
	{
	  assert(p.owner() == P);
	  std::pair<int,int> ad = count(P, state, p.position());
	  return (ad.first*3+ad.second)*PTYPE_SIZE + p.ptype();
	}
	static int value(Piece p, const NumEffectState& state, const Weights& values)
	{
	  return eval::delta(p.owner())*values.value(index(p, state));
	}
	template <Player P>
	static int value(Piece p, const NumEffectState& state, const Weights& values)
	{
	  assert(p.owner() == P);
	  return eval::delta(P)*values.value(index<P>(p, state));
	}
	static int value(int effect_count, PtypeO ptypeo, const Weights& values)
	{
	  assert(effect_count >= 0 && effect_count < 9);
	  int index = effect_count*PTYPE_SIZE + getPtype(ptypeo);
	  return eval::delta(getOwner(ptypeo))*values.value(index);
	}
	static void compile(const Weights& values);

      private:
	template <Player P, Direction D>
	static int update(const NumEffectState& state, Position from, Position moved_to,
			  const PieceMask& ignore, const Weights& values);
	template <Player P, Direction D>
	static int update(const NumEffectState& state, Position from, const Weights& values);
      };
    }
  }
}

#endif /* OSL_EVAL_FROMEFFECT_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
