/* ptypePosition.h
 */
#ifndef OSL_EVAL_PTYPEPOSITION_H
#define OSL_EVAL_PTYPEPOSITION_H

#include "osl/eval/experimental/weights.h"
#include "osl/eval/evalTraits.h"
#include "osl/state/numEffectState.h"

namespace osl
{
  namespace eval
  {
    namespace experimental
    {
      class PtypePosition
      {
      public:
	enum { DIM = 9*5*PTYPE_SIZE };
	
	static int eval(const NumEffectState&, const Weights& values);
	static int evalWithUpdate(const NumEffectState&, Move moved, int last_value, const Weights& values)
	{
	  if (moved.isPass())
	    return last_value;
	  const Player player = moved.player();
	  int result = last_value;
	  if (!moved.isDrop())
	    result -= weight(player, index(player, moved.from(), moved.oldPtype()), values);
	  result += weight(player, index(player, moved.to(), moved.ptype()), values);
	  if (moved.capturePtype() != PTYPE_EMPTY)
	    result -= weight(alt(player), index(alt(player), moved.to(), moved.capturePtype()), values);
	  return result;
	}
	static int pieceValue(const NumEffectState& state, Piece p, const Weights& values);
	
	static int index(Piece p)
	{
	  return index(p.owner(), p.position(), p.ptype());
	}
	static int index(Player player, Position position, Ptype ptype)
	{
	  if (player == WHITE) 
	    position = position.rotate180();
	  return index(position, ptype);
	}
	static int index(Position p, Ptype ptype)
	{
	  int x = p.x();
	  if (x > 5)
	    x = 10-x;
	  --x;				// 0-4
	  int y = p.y() - 1;		// 0-8
	  if (Ptype_Table.getMoveMask(ptype) == Ptype_Table.getMoveMask(GOLD)
	      && y > 3)
	    ptype = GOLD;
	  return (y*5 + x)*PTYPE_SIZE+ptype;
	}

	static int weight(Player owner, int index, const Weights& values)
	{
	  return osl::eval::delta(owner) * values.value(index);      
	}
      }; 
    }
  }
}


#endif /* OSL_EVAL_PTYPEPOSITION_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
