/* pieceEval.h
 */
#ifndef _PIECEEVAL_H
#define _PIECEEVAL_H

#include "osl/eval/ptypeEvalTraits.h"
#include "osl/eval/evalTraits.h"
#include "osl/progress/progress32.h"
#include "osl/progress/progress16.h"
#include "osl/state/numEffectState.h"
#include "osl/misc/carray.h"
#include <cassert>

namespace osl
{
  namespace eval
  {
    class PtypeEvalTable
    {
    private:
      CArray<int, PTYPEO_SIZE> ptypeO2Val;
      CArray<int, PTYPEO_SIZE> promoteVal;
      CArray<int, PTYPEO_SIZE> captureVal;
    public:
      PtypeEvalTable();
      ~PtypeEvalTable();
    private:
      void init();
    public:
      /**
       * ꤫鸫ptypeOζβ
       */
      int value(PtypeO ptypeO) const{
	assert(isValidPtypeO(ptypeO));
	return ptypeO2Val[ptypeO-PTYPEO_MIN];
      }
      /**
       * ptypeζβ
       */
      int value(Ptype ptype) const{
	assert(isValid(ptype));
	return ptypeO2Val[ptype-PTYPEO_MIN];
      }
      /**
       * ptypeOpromoteɾͤ
       */
      int promoteValue(PtypeO ptypeO) const{
	assert(isPromoted(ptypeO));
	return promoteVal[ptypeO-PTYPEO_MIN];
      }
      /**
       * ownerptypeOcapture줿ɾͤ
       */
      int captureValue(PtypeO ptypeO) const{
	assert(isValidPtypeO(ptypeO));
	return captureVal[ptypeO-PTYPEO_MIN];
      }
    };
    extern const PtypeEvalTable Ptype_Eval_Table;

    /**
     * βͥ١ɾؿ.
     * ɬ
     * ͭ +, ͭ -
     *  PtypeEvalTraits<PAWN>::val 
     */
    class PieceEval
    {
      int val;
    public:
      explicit PieceEval(const NumEffectState& state);
      explicit PieceEval(int v) : val(v) {}
      static bool initialized() { return true; }
      void changeTurn() {}
      int value() const
      { 
	assert(isConsistentValueForNormalState<PieceEval>(val)); 
	return val; 
      }
      static int diffWithMove(const NumEffectState&, Move move)
      {
	int ret = 0;
	if (move.capturePtype() != PTYPE_EMPTY)
	  ret += Ptype_Eval_Table.captureValue(move.capturePtypeO());
	if (move.isPromote())
	  ret+=Ptype_Eval_Table.promoteValue(move.ptypeO());
	return ret;
      }
      static int infty() { return 57984; }

      /** 
       * move ˤ֤ͤѲ (SOMA)
       *
       * - move Υޥͤ
       * - ܤϲͤξĴ٤
       * - ֤˴ؤơPROMOTE ̵ͭϹͤƤʤ
       * - ߡ֤move θˤΤӤʤ
       * - ?? ̣ζӱۤͤ
       * - ͤβͤ Player ˤȤäơ
       * - ʤɤϵˤʤ
       */
      template<Player P>
      static int computeDiffAfterMove(const NumEffectState& state,Move move);
      static int computeDiffAfterMove(const NumEffectState& state,Move move)
      {
	assert(state.getTurn() == move.player());
	if (state.getTurn() == BLACK)
	  return computeDiffAfterMove<BLACK>(state,move);
	else
	  return computeDiffAfterMove<WHITE>(state,move);
      }
      /**
       * ¸ΨõѼ֤
       * 
       * ߤζ̤ɾͤ move ζ̤κʬ(P礬Ȥ
       * 褦)֤
       */
      template<Player P>
      static int computeDiffAfterMoveForRP(const NumEffectState& state,Move move)
      {
	assert(move.player()==P);
	const int diff = computeDiffAfterMove<P>(state,move);
	return (P==BLACK) ? diff : -diff;
      }
      static int computeDiffAfterMoveForRP(const NumEffectState& state, Move move)
      {
	if (move.player()==BLACK)
	  return computeDiffAfterMoveForRP<BLACK>(state,move);
	else
	  return computeDiffAfterMoveForRP<WHITE>(state,move);
      }
    private:
      void addVal(int d) { val+=d; }
    public:
      /** state moveؤɾͤͽ¬ */
      int expect(const NumEffectState& /*state*/, Move move) const
      {
	if (move.isPass() || move.isDrop())
	  return value();
	const PtypeO ptypeO=move.ptypeO();
	const PtypeO captured=move.capturePtypeOSafe();
	int result = val 
	  + Ptype_Eval_Table.value(ptypeO)
	  - Ptype_Eval_Table.value(move.oldPtypeO());
	if (getPtype(captured) != PTYPE_EMPTY)
	  result += Ptype_Eval_Table.value(osl::captured(captured))
	    - Ptype_Eval_Table.value(captured);
	return result;
      }

      const Progress32 progress32() const { return Progress32(0); }
      const Progress16 progress16() const { return Progress16(0); }
      static int seeScale() { return 1; }
      /**
       * QuiescenceSearch λ޴ǻ
       */
      static int captureValue(PtypeO ptypeO)
      {
	return Ptype_Eval_Table.captureValue(ptypeO);
      }
      static int value(PtypeO ptypeO)
      {
	return Ptype_Eval_Table.value(ptypeO);
      }

      void update(const NumEffectState& /*new_state*/, Move last_move)
      {
	if (last_move.isPass() || last_move.isDrop())
	  return;

	addVal(Ptype_Eval_Table.value(last_move.ptypeO())
	       - Ptype_Eval_Table.value(last_move.oldPtypeO()));
	if (last_move.capturePtype() != PTYPE_EMPTY) {
	  const PtypeO capture_ptypeo = last_move.capturePtypeO();
	  addVal(Ptype_Eval_Table.value(captured(capture_ptypeo))
		 - Ptype_Eval_Table.value(capture_ptypeo));
	}
      }
      const PieceMask pins(Player) const { return PieceMask(); }
    };

  } // namespace eval
  using eval::PieceEval;
} // namespace osl

#endif /* _PIECEEVAL_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
