#ifndef MISC_BITOP_H
#define MISC_BITOP_H

#include <cassert>
namespace osl
{
  namespace misc
  {
  /**
   * x86 bsf ̿
   */
  template <class Integer> struct Bsf;

  template <> 
  struct Bsf<unsigned int>
  {
    static int bsf(unsigned int mask)
    {
      assert(mask);
#ifdef __x86_64
      int ret;
      __asm__("bsfl %1,%0" : "=r"(ret) : "r"(mask));
      return ret;
#else /* ʲ 32bit Ķ */
#  ifdef i386
      int ret;
      __asm__("bsfl %1,%0" : "=r"(ret) : "r"(mask));
      return ret;
#  else
      return __builtin_ctzl(mask);
#  endif
#endif
    }
  };
  
  template <> 
  struct Bsf<unsigned long long>
  {
    static int bsf(unsigned long long mask)
    {
      assert(mask);
#ifdef __x86_64
      long long ret;
      __asm__("bsfq %1,%0" : "=r"(ret) : "r"(mask));
      return static_cast<int>(ret);
#else /* ʲ 32bit Ķ */
      unsigned int mask32 = static_cast<unsigned int>(mask);
      if (mask32)
	return Bsf<unsigned int>::bsf(mask32);
      mask32 = static_cast<unsigned int>(mask >> 32);
      return 32+Bsf<unsigned int>::bsf(mask32);
#endif
    }
  };
  
  struct BitOp
  {
    template <class Integer>
    static int bsf(Integer mask)
    {
      return Bsf<Integer>::bsf(mask);
    }
    template <class Integer>
    static int takeOneBit(Integer& mask){
      assert(mask);
      const int num=bsf(mask);
      mask &= mask-1;
      return num;
    }

    template <class Integer>
    static int countBit(Integer mask)
    {
      int count=0;
      while (mask) 
      {
	++count;
	mask &= (mask-1);
      }
      return count;
    }
    template <class Integer>
    static bool hasMultipleBit(Integer mask){
      assert(mask);
      return (mask & (mask-1));
    }
    /**
     * non-zeromasksetƤӥåȤLSBõΥӥåȤset줿mask֤.
     */
    template <class Integer>
    static Integer lowestBit(Integer mask){
      assert(mask);
      return static_cast<Integer>(mask & (-mask));
    }
  };
  
#if 0
  /**
   * bit¿?
   * population count
   * Cray, CDC Cyber series, Alliant FX/80.
   * UltraSPARC, Alpha 21264
   * Intel x86 has the parity of a byte
   *#define bitcount(n)                                                     \
   * (tmp = n - ((n >> 1) & 033333333333) - ((n >> 2) & 011111111111), \
   * tmp = ((tmp + (tmp >> 3)) & 030707070707),                        \
   * tmp =  (tmp + (tmp >> 6)),                                        \
   * tmp = (tmp + (tmp >> 12) + (tmp >> 24)) & 077)
   *
   */
  inline int countBitDense(unsigned int mask)
  {
    mask = ((mask>>1)&0x55555555)+(mask&0x55555555);
    mask = ((mask>>2)&0x33333333)+(mask&0x33333333);
    mask = ((mask>>4)+mask)&0xf0f0f0f;
    mask = (mask>>8)+mask;
    return ((mask>>16)+mask)&0x3f;
  }
#endif
  } // namespace misc
} // namespace osl
#endif // _BITOP_H
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
