/* ntesukiSearcher.h
 */
#ifndef _NTESUKI_SEACHER_H
#define _NTESUKI_SEACHER_H
#include "osl/state/numEffectState.h"
#include "osl/ntesuki/ntesukiTable.h"
#include "osl/ntesuki/ntesukiRecord.h"
#include "osl/ntesuki/ntesukiMoveGenerator.h"

#include "osl/ntesuki/ntesukiExceptions.h"
#include "osl/container/moveVector.h"
#include "osl/ntesuki/ntesukiSimulationSearcher.h"

#include <osl/stl/vector.h>

namespace osl
{
  namespace ntesuki
  {
    class
    NtesukiSearcher
    {
    public:
      typedef NumEffectState State;

    private:
      State& state;

      /** . */
      NtesukiMoveGenerator *mg;
      
      /** ȥ󥹥ݥơ֥. */
      NtesukiTable table;

      /** ߥ졼õ. */
      NtesukiSimulationSearcher simulator;

      /** ߤޤǤ˲Ρɤ. */
      unsigned int node_count;

      /** 粿Ρɤޤɤफ. */
      unsigned int read_node_limit;
      
      /** вɤޤɽ뤫. */
      bool verbose;

      /** õǶŪ˽λ뤿 flag. */
      volatile int *stop_flag;

      /** 롼Ȥ鸽õΥΡɤޤǤ path  hash. */
      PathEncoding path;

      /** 롼Ȥ鸽õΥΡɤޤǤ move. */
      typedef std::vector<Move> moves_t;
      moves_t moves_played;
      
      /** 롼Ȥ鸽õΥΡɤޤǤζ. */
      typedef std::vector<NtesukiRecord *> nodes_t;
      nodes_t nodes_played;

      /** ˥ */
      unsigned int max_pass;


    public:
      /**
       * õΤդޤ椹ѿ.
       */
      static bool delay_non_pass;
      static bool ptt_invalid_defense;

      static bool delay_interpose;
      static bool delay_nopromote;
      static bool delay_non_attack;
      static bool read_attack_only;
      static bool ptt_non_attack;

      static bool ptt_siblings_fail;
      static bool ptt_siblings_success;

      static bool ptt_uncle;
      static bool ptt_aunt;

      static unsigned int dynamic_widening_width;

    private:
      /** Iterative widening  scheme */
      NtesukiRecord::IWScheme iwscheme;
      /** Player selection  scheme */
      NtesukiRecord::PSScheme psscheme;
      /** Inversion searching  scheme */
      NtesukiRecord::ISScheme isscheme;

      /** ͤʼ˴ؤ cost */
      int tsumero_cost;

      /** ͤʼ˴ؤͽ¬ */
      int tsumero_estimate;

      /** GC κݤ˥ơ֥ ɤޤǾ뤫 */
      double gc_ratio;

      template<class Search,Player T> class AttackHelper;
      template<class Search,Player T> class DefenseHelper;
      template<class Search,Player T> class CallSimulationAttack;
      template<class Search,Player T> class CallSimulationDefense;
      template<class Search,Player T> class CallSimulationDefenseDisproof;

      /* statistic information */
      unsigned int blockByAttackBack;
      unsigned int blockByPass;
      unsigned int attack_node_count;
      unsigned int attack_node_under_attack_count;
      unsigned int attack_node_moves_count;
      unsigned int defense_node_count;
      unsigned int defense_node_under_attack_count;
      unsigned int defense_node_moves_count;
      unsigned int pass_count, pass_success_count;
      unsigned int pass_attack_count, pass_attack_success_count;
      unsigned int sibling_defense_count, sibling_defense_success_count;
      unsigned int sibling_attack_count, sibling_attack_success_count;
      unsigned int isshogi_defense_count, isshogi_defense_success_count;
      unsigned int isshogi_attack_count, isshogi_attack_success_count;
      unsigned int immediate_win, immediate_lose;
      unsigned int attack_back_count;
      unsigned int proof_without_inversion_count, proof_AND_count, disproof_by_inversion_count;

    public:
      static const int NtesukiNotFound = -1;
      static const int ReadLimitReached = -2;
      static const int TableLimitReached = -3;
    private:
      static const unsigned int INITIAL_PROOF_LIMIT =
      ProofDisproof::PROOF_LIMIT / 4;
      static const unsigned int INITIAL_DISPROOF_LIMIT =
      ProofDisproof::DISPROOF_LIMIT / 4;

      /**
       * ¦ν
       */
      template <Player T>
      NtesukiResult attack(NtesukiRecord* record,
			   const NtesukiRecord* oracle_attack,
			   const NtesukiRecord* oracle_defense,
			   unsigned int proofLimit,
			   unsigned int disproofLimit,
			   int pass_left,
			   const Move last_move);
      template <Player T>
      void attackWithOrder(NtesukiRecord* record,
			   const NtesukiRecord* oracle_attack,
			   const NtesukiRecord* oracle_defense,
			   unsigned int proofLimit,
			   unsigned int disproofLimit,
			   int pass_left,
			   const Move last_move);

      template <Player T>
      NtesukiMove* selectMoveAttack(NtesukiRecord* record,
				    unsigned int& best_proof,
				    unsigned int& sum_disproof,
				    unsigned int& second_proof,
				    unsigned int& best_disproof,
				    unsigned int& step_cost,
				    NtesukiMoveList& moves,
				    const int pass_left);

      /**
       * ɸ˴ؤ׻
       */
      template <Player T>
      NtesukiResult defense(NtesukiRecord* record,
			    const NtesukiRecord* oracle_attack,
			    const NtesukiRecord* oracle_defense,
			    unsigned int proofLimit,
			    unsigned int disproofLimit,
			    int pass_left,
			    const Move last_move);


      template <Player T>
      void defenseWithPlayer(NtesukiRecord* record,
			     const NtesukiRecord* oracle_attack,
			     const NtesukiRecord* oracle_defense,
			     unsigned int proofLimit,
			     unsigned int disproofLimit,
			     int pass_left,
			     const Move last_move);

      template <Player T>
      NtesukiMove* selectMoveDefense(NtesukiRecord* record,
				     unsigned int& best_disproof,
				     unsigned int& sum_proof,
				     unsigned int& second_disproof,
				     unsigned int& best_proof,
				     unsigned int& step_cost,
				     NtesukiMoveList& moves,
				     const int pass_left,
				     const Move last_move);

      /**
       * ֤ǡ꤬ Success Ȥ狼ä硤
       * ¾μƱͤ Success ˤʤʤ Simulaition Ĵ٤.
       *  $\lambda^n$ õξˤϡ§ $\lambda^(n-1)$ move ξˤĴ٤ʤ.
       *
       * e.g. ɬõˤϲĴ٤ʤ.
       */
      template <Player T> void simulateSiblingsSuccess(NtesukiRecord *record,
						       NtesukiRecord *record_best,
						       int pass_left,
						       unsigned int& success_count,
						       unsigned int& total_count);

      /**
       * ֤ǡ꤬ Fail Ȥ狼ä硤
       * ¾μƱͤ Fail ˤʤʤ Simulaition Ĵ٤.
       */
      template <Player T> void simulateSiblingsFail(NtesukiRecord *record,
						    NtesukiRecord *record_best,
						    int pass_left,
						    unsigned int& success_count,
						    unsigned int& total_count);
      /**
       * ˤʤäƤʤ.
       */
      template <Player T> void handleNonAttack(NtesukiRecord *record,
					       int pass_left);
      /**
       * ܻबʤĴ٤.
       */
      template <Player T> void handleTonshi(NtesukiRecord *record,
					    int pass_left,
					    const Move last_move);
      /**
       * ̵̹䤬̵̹礬Ĵ٤.
       */
      template <Player T> void handleInterpose(NtesukiRecord *record,
					       int pass_left);

    public:
      /* ======================
       * ƤФؿ
       */
      NtesukiSearcher(State& state,
		      NtesukiMoveGenerator *mg,
		      unsigned int table_limit,
		      volatile int *stop_flag,
		      bool verbose,
		      int maxPass = NtesukiRecord::SIZE,
		      NtesukiRecord::IWScheme iwscheme = NtesukiRecord::pn_iw,
		      NtesukiRecord::PSScheme psscheme = NtesukiRecord::no_ps,
		      NtesukiRecord::ISScheme isscheme = NtesukiRecord::no_is,
		      int tsumero_cost = 0,
		      int tsumero_estimate = 0,
		      double gc_ratio = 0.33);
      ~NtesukiSearcher();


      template <Player T> int search();

      int searchSlow(Player attacker, int rnl = 160000)
      {
	read_node_limit = rnl;
	if (attacker == BLACK)
	  return search<BLACK>();
	else
	  return search<WHITE>();
      }

      NtesukiTable& getTable();
      int getNodeCount() const { return node_count; }
      bool exceedReadNodeLimit() const {return node_count > read_node_limit;}

    };
  } //ntesuki
} //osl

#endif /* _NTESUKI_SEACHER_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
