/* king8Info.cc
 */
#include "osl/checkmate/king8Info.h"
#include "osl/state/numEffectState.h"
#include "osl/effect_util/additionalEffect.h"
#include "osl/effect_util/pin.h"
#include <bitset>
#include <iostream>

#ifndef MINIMAL
std::ostream& osl::checkmate::operator<<(std::ostream& os, King8Info info)
{
  typedef std::bitset<8> bs_t;
  os << bs_t(info.moveCandidate()) << " " 
     << bs_t(info.libertyCandidate()) << " " 
     << bs_t(info.liberty()) << " " 
     << bs_t(info.dropCandidate());
  return os;
}
#endif
namespace osl
{
  namespace
  {
/**
 * Pζ̤pinƤʳ.
 * @param state - (alt(P)μ֤Ȥϸ¤ʤ)
 * @param target - Pζ̤ΰ
 * @param pos - ̾(Pζ̤Ĺΰ֤ˤȤϸ¤ʤ)
 * @param pinned - pinƤPζmask
 */
template<Player P> inline
bool hasEnoughEffect(NumEffectState const& state,Position target,Position pos, const PieceMask& pinned,
  const PieceMask& on_board_defense)
{
  assert(state.getKingPosition(P)==target);
  assert(pos.isOnBoard());
  PieceMask pieceMask = state.getEffect(pos)&on_board_defense;
  if(pieceMask.none()) return false;
  PieceMask pieceMask1=pieceMask&~pinned;
  if(pieceMask1.any()) return true;
  pieceMask&=pinned;
  while(pieceMask.any()){
    int num=pieceMask.takeOneBit();
    Piece p=state.getPieceOf(num);
    assert(p.isOnBoardByOwner(P));
    Position pos1=p.position();
    assert(Board_Table.getShortOffset(Offset32(pos,target))
	   == pos-target);
    if(Board_Table.getShortOffset(Offset32(pos1,target))==
       pos-target) return true;
  }
  return false;
}
  }
}

template<osl::Player P,osl::Direction Dir>
unsigned int osl::checkmate::
King8Info::hasEffectMask(NumEffectState const& state,Position target, PieceMask pinned,
			 PieceMask on_board_defense)
{
  const Player altP=PlayerTraits<P>::opponent;
  Position pos=target-DirectionPlayerTraits<Dir,P>::offset();
  Piece p=state.getPieceAt(pos);
  if(p.canMoveOn<altP>()){ // ʬζ𤫶
    if(!state.hasEffectBy(P,pos)) // ʬʤ
      return 0x10100u<<static_cast<int>(Dir);
  }
  else if(p.isEdge())
    return 0u;
  const bool has_enough_effect = hasEnoughEffect<altP>(state,target,pos,pinned,on_board_defense);
  if(p.canMoveOn<altP>()){ // ʬζ𤫶
    if(p.isEmpty() &&
       !has_enough_effect){ // ʤ
      // ֤ζΤ褦ˡɲʤȼ夤
      if(state.countEffect(P,pos)>=2 ||
	 effect_util::AdditionalEffect::hasEffect(state,pos,P))
	return 0x1010001u<<static_cast<int>(Dir);
      else
	return 0x10001u<<static_cast<int>(Dir);
    }
    else{
      return 0x10000u<<static_cast<int>(Dir);
    }
  }
  else {
    assert(!p.isEdge());
    if(has_enough_effect) return 0;
    if(state.countEffect(P,pos)>=2 ||
       effect_util::AdditionalEffect::hasEffect(state,pos,P))
      return 0x1000000u<<static_cast<int>(Dir);
    else
      return 0;
  }
  return 0u;
}

template<osl::Player P>
const osl::checkmate::King8Info 
#if (defined __GNUC__) && (! defined GPSONE)
__attribute__ ((noinline))
#endif
osl::checkmate::King8Info::make(NumEffectState const& state,Position target, PieceMask pinned)
{
  PieceMask on_board_defense=state.getOnBoardMask(alt(P));
  on_board_defense.reset(KingTraits<PlayerTraits<P>::opponent>::index);
  unsigned int canMoveMask=hasEffectMask<P,UR>(state,target,pinned,on_board_defense);
  canMoveMask|=hasEffectMask<P,R>(state,target,pinned,on_board_defense);
  canMoveMask|=hasEffectMask<P,DR>(state,target,pinned,on_board_defense);
  canMoveMask|=hasEffectMask<P,U>(state,target,pinned,on_board_defense);
  canMoveMask|=hasEffectMask<P,D>(state,target,pinned,on_board_defense);
  canMoveMask|=hasEffectMask<P,UL>(state,target,pinned,on_board_defense);
  canMoveMask|=hasEffectMask<P,L>(state,target,pinned,on_board_defense);
  canMoveMask|=hasEffectMask<P,DL>(state,target,pinned,on_board_defense);
  return King8Info(canMoveMask);
}

template <osl::Player P>
const osl::checkmate::King8Info 
#if (defined __GNUC__) && (! defined GPSONE)
__attribute__ ((noinline))
#endif
osl::checkmate::
King8Info::make(NumEffectState const& state, Position target)
{
  return make<P>(state,target,state.pin(alt(P)));
}

const osl::checkmate::King8Info osl::checkmate::
King8Info::make(Player attack, NumEffectState const& state)
{
  const Position king=state.getKingPosition(alt(attack));
  if (attack == BLACK)
    return make<BLACK>(state, king);
  else
    return make<WHITE>(state, king);
}

const osl::checkmate::King8Info osl::checkmate::
King8Info::makeWithPin(Player attack, NumEffectState const& state,
		       const PieceMask& pins)
{
  const Position king=state.getKingPosition(alt(attack));
  if (attack == BLACK)
    return make<BLACK>(state, king, pins);
  else
    return make<WHITE>(state, king, pins);
}

osl::checkmate::EdgeTable::EdgeTable()
{
  edge_mask.fill(~0u);
  for (int x=1; x<=9; ++x) {
    for (int y=1; y<=9; ++y) {
      Position king(x,y);
      for (int d=DIRECTION_MIN; d<=DR; ++d) {
	Position target = king+Board_Table.getOffset(BLACK, Direction(d));
	if (target.x() <= 1 || target.x() >= 9 || target.y() <=1 || target.y() >=9)
	  edge_mask[BLACK][king.index()] &= ~(0x100<<d);
	target = king+Board_Table.getOffset(WHITE, Direction(d));
	if (target.x() <= 1 || target.x() >= 9 || target.y() <=1 || target.y() >=9)
	  edge_mask[WHITE][king.index()] &= ~(0x100<<d);
      }
    }
  }
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:

