/* fromEffect.cc
 */
#include "osl/eval/experimental/fromEffect.h"
#include "osl/eval/pieceEval.h"
#include <iostream>


std::pair<int,int> osl::eval::experimental::
FromEffect::count(Player me, const NumEffectState& state, Position position) 
{
  int attack = 0, defense = 0;
  assert(position.isOnBoard());
  const Player attacker = alt(me);
  attack = std::min(2, state.countEffect(attacker, position));
  defense = std::min(2, state.countEffect(alt(attacker), position));
  return std::make_pair(attack, defense);
}

int osl::eval::experimental::
FromEffect::pieceValue(const NumEffectState& state, Piece p, const Weights& values)
{
  if (p.position().isPieceStand() || p.ptype() == KING)
    return 0;
  return  value(p, state, values);
}


int osl::eval::experimental::
FromEffect::eval(const NumEffectState& state, const Weights& values)
{
  int result = 0;
#if 0
  for (int i=0; i<Piece::SIZE; ++i) {
    Piece p = state.getPieceOf(i);
    if (p.position().isPieceStand() || p.ptype() == KING)
      continue;
    result += value(p, state, values);
  }
#else
  PieceMask black = state.getOnBoardMask(BLACK);
  while (black.any()) {
    const int i = black.takeOneBit();
    const Piece p = state.getPieceOf(i);
    if (p.ptype() == KING)
      continue;
    result += value<BLACK>(p, state, values);
    
  }
  PieceMask white = state.getOnBoardMask(WHITE);
  while (white.any()) {
    const int i = white.takeOneBit();
    const Piece p = state.getPieceOf(i);
    if (p.ptype() == KING)
      continue;
    result += value<WHITE>(p, state, values);
    
  }
#endif
  return result;
}

template <osl::Player P, osl::Direction D>
int osl::eval::experimental::
FromEffect::update(const NumEffectState& state, Position from, Position moved_to,
		   const PieceMask& ignore, const Weights& values) 
{
  const Position to = from + DirectionPlayerTraits<D,P>::offset();
  if (to == moved_to) 	// only valid for moves without capture
    return 0;
  // std::cerr << P << D << " " << state << from << " " << moved_to << " " << to << "\n" << std::flush;
  const Piece p = state.getPieceAt(to);
  if (! p.isPiece() || p.ptype() == KING)
    return 0;

  const int me = state.countEffect(P, to, ignore);
  if (me >= 2)
    return 0;
  int result = 0;
  const int op = std::min(2, state.countEffect(alt(P), to));
  if (p.owner() == P) {
    result -= value(op*3+me+1, p.ptypeO(), values);
    result += value(op*3+me,   p.ptypeO(), values);      
  }
  else {
    result -= value((me+1)*3+op,p.ptypeO(), values);
    result += value(me*3+op,    p.ptypeO(), values);      
  }
  return result;
}  

template <osl::Player P, osl::Direction D>
int osl::eval::experimental::
FromEffect::update(const NumEffectState& state, Position from, const Weights& values) 
{
  const Position to = from + DirectionPlayerTraits<D,P>::offset();
  // std::cerr << P << D << " " << state << from << " " << to << "\n" << std::flush;
  const Piece p = state.getPieceAt(to);
  if (! p.isPiece() || p.ptype() == KING)
    return 0;
  const int me = state.countEffect(P, to);
  assert(me > 0);
  if (me > 2)
    return 0;
  int result = 0;
  const int op = std::min(2, state.countEffect(alt(P), to));
  if (p.owner() == P) {
    result -= value(op*3+me-1,p.ptypeO(), values);
    result += value(op*3+me,  p.ptypeO(), values);      
  }
  else {
    result -= value((me-1)*3+op,p.ptypeO(), values);
    result += value(me*3+op,    p.ptypeO(), values);      
  }
  return result;
}

int osl::eval::experimental::
FromEffect::evalWithUpdate(const NumEffectState& state, Move moved, int last_value, const Weights& values)
{
  if (moved.player() == BLACK)
    return evalWithUpdate<BLACK>(state, moved, last_value, values);
  else
    return evalWithUpdate<WHITE>(state, moved, last_value, values);
}

template <osl::Player player>
int osl::eval::experimental::
FromEffect::evalWithUpdate(const NumEffectState& state, Move moved, int last_value,
			   const Weights& values)
{
  // std::cerr << state << moved << "\n";
  assert(player == moved.player());
  if (moved.isPass())
    return last_value;
  if (Ptype_Table.hasLongMove(moved.ptype()) || Ptype_Table.hasLongMove(moved.oldPtype()))
    return eval(state, values);
  NumBitmapEffect effect = state.getEffect(moved.to());
  if ((moved.capturePtype() != PTYPE_EMPTY)
      || (effect.getMask(1) & mask_t::makeDirect(PtypeFuns<LANCE>::indexMask)).any()
      || (effect.getMask(1) & mask_t::makeDirect(PtypeFuns<ROOK>::indexMask)).any()
      || (effect.getMask(1) & mask_t::makeDirect(PtypeFuns<BISHOP>::indexMask)).any())
    return eval(state, values);
  int result = last_value;
  if (! moved.isDrop()) {
    NumBitmapEffect effect = state.getEffect(moved.from());
    if ((effect.getMask(1) & mask_t::makeDirect(PtypeFuns<LANCE>::indexMask)).any()
	|| (effect.getMask(1) & mask_t::makeDirect(PtypeFuns<ROOK>::indexMask)).any()
	|| (effect.getMask(1) & mask_t::makeDirect(PtypeFuns<BISHOP>::indexMask)).any())
      return eval(state, values);
    PieceMask ignore;
    ignore.set(state.getPieceAt(moved.to()).number());
    if (moved.ptype() != KING)
    {
      const int me = std::min(2, state.countEffect(player, moved.from(), ignore));
      const int op = std::min(2, state.countEffect(alt(player), moved.from()));
      result -= value(op*3+me, moved.oldPtypeO(), values);
    }
    switch (moved.oldPtype()) {
    case KNIGHT:
      result += update<player, UUL>(state, moved.from(), moved.to(), ignore, values);
      result += update<player, UUR>(state, moved.from(), moved.to(), ignore, values);
      break;
    case SILVER:
      result += update<player, U>(state, moved.from(), moved.to(), ignore, values);
      result += update<player, UL>(state, moved.from(), moved.to(), ignore, values);
      result += update<player, UR>(state, moved.from(), moved.to(), ignore, values);
      result += update<player, DL>(state, moved.from(), moved.to(), ignore, values);
      result += update<player, DR>(state, moved.from(), moved.to(), ignore, values);
      break;
    case KING:
      result += update<player, DL>(state, moved.from(), moved.to(), ignore, values);
      result += update<player, DR>(state, moved.from(), moved.to(), ignore, values); // fall through
    case GOLD: case PPAWN: case PLANCE: case PKNIGHT: case PSILVER:
      result += update<player, UL>(state, moved.from(), moved.to(), ignore, values);
      result += update<player, UR>(state, moved.from(), moved.to(), ignore, values);
      result += update<player, L>(state, moved.from(), moved.to(), ignore, values);
      result += update<player, R>(state, moved.from(), moved.to(), ignore, values);
      result += update<player, D>(state, moved.from(), moved.to(), ignore, values); // fall through
    case PAWN:
      result += update<player, U>(state, moved.from(), moved.to(), ignore, values);
      break;
    default:
      assert(0);
    }
  }

  // to
  if (moved.ptype() != KING)
  {
    result += value(state.getPieceOnBoard(moved.to()), state, values);
  }

  switch (moved.ptype()) {
  case KNIGHT:
    result += update<player, UUL>(state, moved.to(), values);
    result += update<player, UUR>(state, moved.to(), values);
    break;
  case SILVER:
    result += update<player, U>(state, moved.to(), values);
    result += update<player, UL>(state, moved.to(), values);
    result += update<player, UR>(state, moved.to(), values);
    result += update<player, DL>(state, moved.to(), values);
    result += update<player, DR>(state, moved.to(), values);
    break;
  case KING:
    result += update<player, DL>(state, moved.to(), values);
    result += update<player, DR>(state, moved.to(), values); // fall through
  case GOLD: case PPAWN: case PLANCE: case PKNIGHT: case PSILVER:
    result += update<player, UL>(state, moved.to(), values);
    result += update<player, UR>(state, moved.to(), values);
    result += update<player, L>(state, moved.to(), values);
    result += update<player, R>(state, moved.to(), values);
    result += update<player, D>(state, moved.to(), values); // fall through
  case PAWN:
    result += update<player, U>(state, moved.to(), values);
    break;
  default:
    assert(0);
  }
  return result;
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
