/* midgameEval.cc
 */
#include "osl/eval/experimental/midgameEval.h"
#include "osl/eval/experimental/ptypePosition.h"
#include "osl/eval/experimental/piecePair.h"
#include "osl/eval/experimental/kingArea.h"
#include "osl/progress/effect5x3.h"
#include "osl/oslConfig.h"
#include <iostream>
#include <iomanip>
#include <fstream>

namespace osl
{
  using namespace osl::eval::experimental;
  namespace
  {
    bool initialized_flag = false;
    typedef PiecePair f0;
    typedef PtypePosition f1;
    typedef KingArea f2;
    CArray<Weights,3> weights;

    CArray<size_t,3> dimensions = {{
      f0::DIM, f1::DIM, f2::DIM,
    }};
  }
}

#define UPDATE(id, new_state, last_move, old_value) f##id::evalWithUpdate(new_state, last_move, old_value, weights[id])
#define UPDATEC(id, new_state, last_move, old_value) f##id::evalWithUpdateCompiled(new_state, last_move, old_value)
#define EVAL(id, state) f##id::eval(state, weights[id])
#define PIECEVALUE(id, state, p) f##id::pieceValue(state, p, weights[id])

osl::eval::experimental::
MidgameEval::MidgameEval(const NumEffectState& state)
  : piece(state)
{
  assert(values.size() == dimensions.size());
  assert(initialized());
  values[0]  = EVAL(0, state);
  values[1]  = EVAL(1, state);
  values[2]  = EVAL(2, state);
}

bool osl::eval::experimental::
MidgameEval::initialized()
{
  return initialized_flag;
}

bool osl::eval::experimental::
MidgameEval::setUp(const char *filename)
{
  if (initialized_flag)
    return true;

  PiecePair::init();
  
  std::ifstream is(filename);
  int dummy_for_piece;
  is >> dummy_for_piece;
  
  for (size_t i=0; i<dimensions.size(); ++i) {
    weights[i].resetDimension(dimensions[i]);
    for (size_t j=0; j<dimensions[i]; ++j) {
      int val;
      is >> val;
      weights[i].setValue(j, val);
    }
  }

  PiecePair::sanitize(weights[0]);
  PiecePair::compile(weights[0]);
  initialized_flag = is;
  if (! is)
    std::cerr << "warning read weights failed in MidgameEval::setUp " << filename << "\n";
  return is;
}

bool osl::eval::experimental::
MidgameEval::setUp()
{
  std::string filename = OslConfig::home();
  filename += "/data/experimental-eval.txt";
  return setUp(filename.c_str());  
}

void osl::eval::experimental::
MidgameEval::update(const NumEffectState& new_state, Move last_move)
{
  piece.update(new_state, last_move);
  values[0]  = UPDATEC(0, new_state, last_move, values[0]);
  values[1]  = UPDATE(1, new_state, last_move, values[1]);
  values[2]  = UPDATE(2, new_state, last_move, values[2]);
}

int osl::eval::experimental::
MidgameEval::pieceValue(const NumEffectState& state, Piece p)
{
  int sum = eval::Ptype_Eval_Table.value(p.ptypeO());
  sum += PIECEVALUE(0, state, p);
  sum += PIECEVALUE(1, state, p);
  sum += PIECEVALUE(2, state, p);

  return sum;
}

void osl::eval::experimental::
MidgameEval::showEvalSummary(const NumEffectState& state)
{
  {
    MidgameEval e(state);
    std::cerr << "value " << e.value() << "   ";
    for (size_t i=0; i<e.values.size(); ++i)
      std::cerr << "  " << e.values[i];
    std::cerr << "\n";
  }

  for (int y=1; y<=9; ++y) {
    for (int x=9; x>=1; --x) {
      const Position position(x,y);
      const Piece piece = state.getPieceOnBoard(position);
      std::cerr << std::setw(4) << pieceValue(state, piece);
    }
    std::cerr << "\n";
  }
}

void osl::eval::experimental::
MidgameEval::setValues(const SimpleState& sstate, PieceValues& out)
{
  NumEffectState state(sstate);
  for (int i=0; i<Piece::SIZE; i++) {
    const Piece target = state.getPieceOf(i);
    out[i] = pieceValue(state, target);
  }  
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
