#include "osl/eval/experimental/mobility.h"
#include "osl/mobility/rookMobility.h"
#include "osl/mobility/bishopMobility.h"
#include "osl/mobility/lanceMobility.h"

using osl::MultiInt;

osl::misc::CArray<MultiInt, 18>
osl::eval::experimental::RookMobility::rook_vertical_table;

osl::misc::CArray<MultiInt, 18>
osl::eval::experimental::RookMobility::rook_horizontal_table;


osl::misc::CArray<MultiInt, 36>
osl::eval::experimental::BishopMobility::bishop_table;


osl::misc::CArray<MultiInt, 9>
osl::eval::experimental::LanceMobility::lance_table;


void osl::eval::experimental::
RookMobility::setUp(const Weights &weights,int stage)
{
  for (size_t i = 0; i < 9; ++i)
  {
    rook_vertical_table[i][stage] = weights.value(i);
  }
  for (size_t i = 0; i < 9; ++i)
  {
    rook_horizontal_table[i][stage] = weights.value(i + 9);
  }
  for (size_t i = 0; i < 9; ++i)
  {
    rook_vertical_table[i+9][stage] = weights.value(i + 18);
  }
  for (size_t i = 0; i < 9; ++i)
  {
    rook_horizontal_table[i+9][stage] = weights.value(i + 27);
  }
}

template <int Sign> inline
void osl::eval::experimental::
RookMobilityAll::adjust(bool promoted, int vertical, int horizontal, MultiInt& value)
{
  int offset=0;
  if(promoted) offset=9;
  if(Sign>0)
    value+= RookMobility::rook_vertical_table[vertical+offset]+
      RookMobility::rook_horizontal_table[horizontal+offset];
  else{ // Sign<0
    value-= RookMobility::rook_vertical_table[vertical+offset]+
      RookMobility::rook_horizontal_table[horizontal+offset];
  }
}

void osl::eval::experimental::
RookMobilityAll::eval(const NumEffectState& state, MultiInt& out)
{
  out.clear();
  for (int i = PtypeTraits<ROOK>::indexMin;
       i < PtypeTraits<ROOK>::indexLimit;
       ++i)
  {
    const Piece rook = state.getPieceOf(i);
    if (! rook.isOnBoard())
      continue;
    if (rook.owner() == BLACK)
    {
      const int vertical = osl::mobility::RookMobility::countVerticalAll<BLACK>(state,i);
      const int horizontal = osl::mobility::RookMobility::countHorizontalAll<BLACK>(
	state, i);
      adjust<1>(rook.isPromoted(), vertical, horizontal, out);
    }
    else
    {
      const int vertical = osl::mobility::RookMobility::countVerticalAll<WHITE>(state,i);
      const int horizontal = osl::mobility::RookMobility::countHorizontalAll<WHITE>(
	state, i);
      adjust<-1>(rook.isPromoted(), vertical, horizontal, out);
    }
  }
}




void osl::eval::experimental::
BishopMobility::setUp(const Weights &weights,int stage)
{
  for (size_t i = 0; i < 18; ++i)
  {
    bishop_table[i][stage] = weights.value(i);
  }
  for (size_t i = 0; i < 18; ++i)
  {
   bishop_table[i+18][stage] = weights.value(i + 18);
  }
}

template <int Sign> inline
void osl::eval::experimental::
BishopMobilityAll::adjust(bool promoted, int mobility, MultiInt& value)
{
  int count=0;
  if(promoted) count=18;
  if(Sign>0)
      value +=BishopMobility::bishop_table[mobility+count];
    else
      value -=BishopMobility::bishop_table[mobility+count];
}
void osl::eval::experimental::
BishopMobilityAll::eval(const NumEffectState& state, MultiInt& out)
{
  out.clear();
  for (int i = PtypeTraits<BISHOP>::indexMin;
       i < PtypeTraits<BISHOP>::indexLimit;
       ++i)
  {
    const Piece bishop = state.getPieceOf(i);
    if (! bishop.isOnBoard())
      continue;
    if (bishop.owner() == BLACK)
    {
      const int mobility = mobility::BishopMobility::countAll<BLACK>(state, i);
      adjust<1>(bishop.isPromoted(), mobility, out);
    }
    else
    {
      const int mobility = mobility::BishopMobility::countAll<WHITE>(state, i);
      adjust<-1>(bishop.isPromoted(), mobility, out);
    }
  }
}



void osl::eval::experimental::
LanceMobility::setUp(const Weights &weights,int stage)
{
  for (size_t i = 0; i < 9; ++i)
  {
    lance_table[i][stage] = weights.value(i);
  }
}

template <int Sign> inline
void osl::eval::experimental::
LanceMobilityAll::adjust(int mobility, MultiInt& value)
{
  if(Sign>0)
    value += LanceMobility::lance_table[mobility];
  else
    value -= LanceMobility::lance_table[mobility];
}

void osl::eval::experimental::
LanceMobilityAll::eval(const NumEffectState &state, MultiInt& out)
{
  out.clear();
  for (int i = PtypeTraits<LANCE>::indexMin;
       i < PtypeTraits<LANCE>::indexLimit;
       ++i)
  {
    const Piece lance = state.getPieceOf(i);
    if (!lance.isOnBoardNotPromoted())
      continue;
    if (lance.pieceIsBlack())
    {
      const int mobility = osl::mobility::LanceMobility::countAll<BLACK>(
	state, lance.position(),i);
      adjust<1>(mobility, out);
    }
    else
    {
      const int mobility = osl::mobility::LanceMobility::countAll<WHITE>(
	state, lance.position(),i);
      adjust<-1>(mobility, out);
    }
  }
}

// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
