/* ptypeAttackedLong.cc
 */
#include "osl/eval/experimental/ptypeAttackedLong.h"
#include "osl/eval/evalTraits.h"

int osl::eval::experimental::
PtypeAttackedLong::index(Piece p, const NumEffectState& state)
{
  const Position target = p.position();
  if (state.hasEffectLong<LANCE>(alt(p.owner()), target))
    return index(p.ptype(), LANCE, state.hasEffectBy(p.owner(), p.position()));
  if (state.hasEffectLong<BISHOP>(alt(p.owner()), target))
    return index(p.ptype(), BISHOP, state.hasEffectBy(p.owner(), p.position()));
  if (state.hasEffectLong<ROOK>(alt(p.owner()), target))
    return index(p.ptype(), ROOK, state.hasEffectBy(p.owner(), p.position()));
  return -1;
}

int osl::eval::experimental::
PtypeAttackedLong::pieceValue(const NumEffectState& state, Piece p, const Weights& values)
{
  if (p.position().isPieceStand())
    return 0;
  const int index = PtypeAttackedLong::index(p, state);
  if (index < 0)
    return 0;
  return eval::delta(p.owner())*values.value(index);
}

int osl::eval::experimental::
PtypeAttackedLong::eval(const NumEffectState& state, const Weights& values)
{
  int result = 0;
#if 0
  for (int i=0; i<Piece::SIZE; ++i) {
    Piece p = state.getPieceOf(i);
    if (p.position().isPieceStand())
      continue;
    const int index = this->index(p, state);
    if (index < 0)
      continue;
    if (p.owner() == BLACK) {
      result += values.value(index);
    } else {
      result -= values.value(index);
    }
  }
#else
  for (int i=PtypeTraits<LANCE>::indexMin; i<PtypeTraits<LANCE>::indexLimit; ++i) {
    const Piece lance = state.getPieceOf(i);
    if (lance.ptype() != LANCE || lance.position().isPieceStand())
      continue;
    Position pos=Board_Table.nextPosition(lance.owner(), lance.position(), U);
    while (state.getPieceAt(pos).isEmpty()) {
      pos = Board_Table.nextPosition(lance.owner(), pos, U);
      const Piece attacked = state.getPieceAt(pos);
      if (attacked.isOnBoardByOwner(alt(lance.owner()))) {
	result += values.value(index(attacked.ptype(), LANCE, state.hasEffectBy(attacked.owner(), attacked.position())))
	  *osl::eval::delta(attacked.owner());
	break;
      }
    }
  }
  CArray<Direction,4> diag = {{ UL, UR, DL, DR }};
  for (int d=0; d<4; ++d) {
    const Direction dir = diag[d];
    for (int i=PtypeTraits<BISHOP>::indexMin; i<PtypeTraits<BISHOP>::indexLimit; ++i) {
      const Piece bishop = state.getPieceOf(i);
      if (bishop.position().isPieceStand())
	continue;
      Position pos=Board_Table.nextPosition(bishop.owner(), bishop.position(), dir);
      while (state.getPieceAt(pos).isEmpty()) {
	pos = Board_Table.nextPosition(bishop.owner(), pos, dir);
	const Piece attacked = state.getPieceAt(pos);
	if (attacked.isOnBoardByOwner(alt(bishop.owner()))) {
	  result += values.value(index(attacked.ptype(), BISHOP, state.hasEffectBy(attacked.owner(), attacked.position())))
	    * osl::eval::delta(attacked.owner());
	  break;
	}
      }
    }
  }
  CArray<Direction,4> vert_horiz = {{ U, D, L, R }};
  for (int d=0; d<4; ++d) {
    const Direction dir = vert_horiz[d];
    for (int i=PtypeTraits<ROOK>::indexMin; i<PtypeTraits<ROOK>::indexLimit; ++i) {
      const Piece rook = state.getPieceOf(i);
      if (rook.position().isPieceStand())
	continue;
      Position pos=Board_Table.nextPosition(rook.owner(), rook.position(), dir);
      while (state.getPieceAt(pos).isEmpty()) {
	pos = Board_Table.nextPosition(rook.owner(), pos, dir);
	const Piece attacked = state.getPieceAt(pos);
	if (attacked.isOnBoardByOwner(alt(rook.owner()))) {
	  result += values.value(index(attacked.ptype(), ROOK, state.hasEffectBy(attacked.owner(), attacked.position())))
	    * eval::delta(attacked.owner());
	  break;
	}
      }
    }
  }
#endif
  return result;
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
