#include "osl/record/ki2.h"
#include "osl/record/ki2IOError.h"
#include "osl/misc/sjis2euc.h"
#include <boost/algorithm/string/trim.hpp>
#include <fstream>
#include <iostream>

namespace osl
{
  namespace record
  {
    namespace // anonymous
    {
      enum ParseResult
      {
        OK = 0,
        KOMAOCHI,
      };

      ParseResult ki2ParseLine(RecordVisitor& rv, 
                               std::string line, 
                               const osl::record::KanjiMove& kmove)
      {
        boost::algorithm::trim(line);

        if (line.empty() || line.at(0) == '*')
          return OK;
        else if (line.size() > 6 && line.substr(0,6) == "ꡧ")
        {
          Record *record = rv.getRecord();
          const std::string player_name(line.substr(7));
          record->setPlayer(osl::BLACK, player_name);
          return OK;
        }
        else if (line.size() > 6 && line.substr(0,6) == "ꡧ")
        {
          Record *record = rv.getRecord();
          const std::string player_name(line.substr(7));
          record->setPlayer(osl::WHITE, player_name);
          return OK;
        }
        else if (line.size() > 8 && line.substr(0,8) == "䡧")
          return KOMAOCHI;
        else if (line.substr(0,2) != "" && line.substr(0,2) != "")
          return OK;
    
        std::string move_str;
        for (size_t i = 0; ; )
        {
          if (i < line.size() && 
              (line.at(i) == ' ' || line.at(i) == '\t'))
          {
            ++i;
            continue;
          }

          if ( (line.substr(i,2) == "" || 
                line.substr(i,2) == "" ||
                i+1 >= line.size())
               && !move_str.empty())
          {
            // apply move_str
            Move last_move;
            if (rv.getLastMove())
              last_move = rv.getLastMove()->getMove();
            const NumEffectState state(*rv.getState());
            const Move move = kmove.strToMove(move_str, state, last_move);
	    rv.addMoveAndAdvance(move);
            move_str.clear();
          }
          if (i+1 >= line.size())
            return OK;
          move_str.append(line.substr(i,2));
          i += 2;
        } // for
      }

    } // anonymous namespace
  } //namespace record
} // namespace osl


osl::record::ki2::
InputStream::InputStream(std::istream& is, bool verbose) 
  : is(is), state(SimpleState(HIRATE))
{
  if (! is)
  {
    std::cerr << "InputStream::InputStream cannot read \n";
    abort();
  }
  kmove.setVerbose(verbose);
}
  
osl::record::ki2::
InputStream::~InputStream(){}
  
void osl::record::ki2::
InputStream::load(Record* rec)
{
  rv.setState(&state);
  rv.setRecord(rec);
  std::string line;
  while (std::getline(is, line)) 
  {
    osl::misc::sjis2euc(line);
    const ParseResult result = ki2ParseLine(rv, line, kmove);
    switch (result)
    {
      case OK:
        continue;
      case KOMAOCHI:
      {
        const std::string msg = "ERROR: Komaochi (handicapped game) records are not available: ";
        std::cerr << msg << "\n";
        throw Ki2IOError(msg);
      }
      default:
        assert(false);
    }
  }
}

osl::record::ki2::
Ki2File::Ki2File(const std::string& fileName, bool verbose)
  : verbose(verbose)
{
  std::ifstream ifs(fileName.c_str());
  if (! ifs)
  {
    const std::string msg = "Ki2File::Ki2File file cannot read ";
    std::cerr << msg << fileName << "\n";
    throw Ki2IOError(msg + fileName);
  }
  InputStream irs(ifs, verbose);
  irs.load(&rec);
}

const osl::record::Record& osl::record::ki2::
Ki2File::getRecord() const
{
  return rec;
}

const osl::NumEffectState osl::record::ki2::
Ki2File::getInitialState() const
{
  return rec.getInitialState();
}
