/* searchMoveSet.cc
 */
#include "osl/search/searchMoveSet.h"
#include "osl/search/simpleHashRecord.h"
#include "osl/record/csa.h"
#include <iostream>

osl::search::
SearchMoveSet::SearchMoveSet(const SearchMoveSet& src)
  : base_t(src)
{
}

osl::search::SearchMoveSet& osl::search::
SearchMoveSet::operator=(const SearchMoveSet& src)
{
  if (this != &src)
    base_t::operator=(src);
  return *this;
}

osl::search::
SearchMoveSet::~SearchMoveSet()
{
  assert(isValidAll());
}

bool osl::search::
SearchMoveSet::isValidAll() const
{
#ifdef OSL_SMP
  Mutex::scoped_lock lk(mutex);
#endif
  for (const_iterator p=begin(); p!=end(); ++p)
  {
    if (! p->getMove().isValidOrPass())
      return false;
  }
  return true;
}

#ifndef MINIMAL
void osl::search::
SearchMoveSet::dump(std::ostream& os) const
{
#ifdef OSL_SMP
  Mutex::scoped_lock lk(mutex);
#endif
  size_t i=0;
  for (const_iterator p=begin(); p!=end(); ++p, ++i)
  {
    os << 'M' << i << "\t" << record::csa::show(p->getMove());
    os << " " << p->getLogProb();
    
    if (p->record)
    {
      os << "\t";
      if (p->record->hasLowerBound(0))
	os <<  p->record->lowerBound();
      else 
	os << "*";
      os << " < ";
      if (p->record->hasUpperBound(0))
	os <<  p->record->upperBound();
      else 
	os << "*";
    }
    os << "\n";
    assert(p->getMove().isValidOrPass());
  }
}
#endif
/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
