/* simpleHashRecord.cc
 */
#include "osl/search/simpleHashRecord.h"
#include "osl/record/csa.h"
#include <map>
#include <iostream>
#include <iomanip>

#ifndef MINIMAL
void osl::search::
SimpleHashRecord::dump(std::ostream& os) const
{
  os << "SimpleHashRecord " << this 
     << " node_count " << nodeCount() << "\n";
  os << "best move " << record::csa::show(best_move.getMove())
	    << " " << best_move.getLogProb()
	    << " " << best_move.record << "\t";
  os << "limit: l " << lower_limit << " u " << upper_limit << "\n";
  os << "in_check " << is_king_in_check << "\n";
  os << "tried oracle " << qrecord.attack_oracle.proof 
     << " " << qrecord.attack_oracle.proof_last_move
     << " " << qrecord.attack_oracle.disproof << "\n";
  if (hasLowerBound(0))
    os <<  lowerBound();
  else 
    os << "*";
  os << " < ";
  if (hasUpperBound(0))
    os <<  upperBound();
  else 
    os << "*";
  os << "\n";
  os << "moves " << limit_of_moves << "\n";
  // moves().dump(os);
  qrecord.dump(os);
}

void osl::search::
SimpleHashRecord::dumpNodeCount(std::ostream& os, size_t max) const
{
  os << "total: " << search_nodes << "\t";
  if (search_nodes) {
    const double total = search_nodes;
    typedef std::multimap<double, SearchMove, std::greater<double> > map_t;
    map_t m;
    SearchMoveSet::const_range r(moves());
    for (SearchMoveSet::const_iterator p=r.first; p!=r.last; ++p)
    {
      if (p->record)
	m.insert(std::make_pair(p->record->nodeCount()/total, *p));
    }
    size_t i=0;
    for (map_t::const_iterator p=m.begin(); p!=m.end() && i < max; ++p, ++i)
    {
      os << std::setprecision(3) << p->first * 100.0
	 << "% " << record::csa::show(p->second.getMove()) << "\t";
    }
  }
  os << std::endl;
}

double osl::search::
SimpleHashRecord::nodeRatio(Move move) const
{
  const SearchMove *recorded = moves().find(move);
  if (! recorded || ! recorded->record)
    return 0.0;
  return recorded->record->nodeCount() / (double)search_nodes;
}
#endif
size_t osl::search::
SimpleHashRecord::sumOfNodeCountOfChildren() const
{
  size_t sum = 0;
  SearchMoveSet::const_range r(moves());
  for (SearchMoveSet::const_iterator p=r.first; p!=r.last; ++p)
  {
    if (p->record)
      sum += p->record->nodeCount();
  }
  return sum;
}

void osl::search::
SimpleHashRecord::setSumNodeCountOfChildren()
{
  search_nodes = sumOfNodeCountOfChildren();
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
