#include "osl/direction.h"
#include "osl/directionTraits.h"
#include "osl/boardTable.h"
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <iostream>
using namespace osl;
extern bool isShortTest;

class directionTest : public CppUnit::TestFixture {
  CPPUNIT_TEST_SUITE( directionTest );
  CPPUNIT_TEST( testTraits);
  CPPUNIT_TEST( testTable);
  CPPUNIT_TEST( testShow );
  CPPUNIT_TEST_SUITE_END();
 public:
  void testTraits();
  void testTable();
  void testShow();
};

CPPUNIT_TEST_SUITE_REGISTRATION(directionTest);

  void directionTest::testTraits(){
    CPPUNIT_ASSERT(DirectionTraitsGen<LONG_D>::blackDx==0);
    CPPUNIT_ASSERT(DirectionTraitsGen<LONG_D>::blackDy==1);
    CPPUNIT_ASSERT(DirectionTraits<LONG_D>::blackOffset()==(Offset(0,1)));
    CPPUNIT_ASSERT((DirectionPlayerTraits<LONG_D,BLACK>::offset())==(Offset(0,1)));
    CPPUNIT_ASSERT((DirectionPlayerTraits<LONG_D,BLACK>::offset())!=Offset::ZERO());
    CPPUNIT_ASSERT((DirectionPlayerTraits<UUR,WHITE>::offset())==(Offset(1,2)));
    CPPUNIT_ASSERT((DirectionPlayerTraits<UUR,WHITE>::offset())==Offset::makeDirect(18));
  }

  void directionTest::testTable(){
    CPPUNIT_ASSERT_EQUAL(Board_Table.getShortOffset(Offset32(6,0)),
			 Offset(1,0));
    CPPUNIT_ASSERT(DirectionTraitsGen<LONG_D>::blackDy==1);
    CPPUNIT_ASSERT(DirectionTraits<LONG_D>::blackOffset()==(Offset(0,1)));
    CPPUNIT_ASSERT((DirectionPlayerTraits<LONG_D,BLACK>::offset())==(Offset(0,1)));
    CPPUNIT_ASSERT((DirectionPlayerTraits<LONG_D,BLACK>::offset())!=Offset::ZERO());
  }

template<int i>
static void showDirect(){
  Direction d=static_cast<Direction>(i);
  std::cerr << i << "," << d << "," << DirectionTraits<static_cast<Direction>(i)>::mask << std::endl;
}
void directionTest::testShow(){
  if (! isShortTest)
    {
      showDirect<0>();
      showDirect<1>();
      showDirect<2>();
      showDirect<3>();
      showDirect<4>();
      showDirect<5>();
      showDirect<6>();
      showDirect<7>();
      showDirect<8>();
      showDirect<9>();
      showDirect<10>();
      showDirect<11>();
      showDirect<12>();
      showDirect<13>();
      showDirect<14>();
      showDirect<15>();
      showDirect<16>();
      showDirect<17>();
      for(int dy=-8;dy<=8;dy++){
	for(int dx=-8;dx<=8;dx++){
	  std::cerr << "LongDirection<BLACK>(" << dx << "," << dy << ")=" << Board_Table.getLongDirection<BLACK>(Offset32(dx,dy)) << std::endl;
	}
      }
    }
}
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
