/* midgameEval.t.cc
 */
#include "osl/eval/experimental/midgameEval.h"
#include "osl/state/numEffectState.h"
#include "../consistencyTest.h"

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <iostream>
#include <fstream>

class MidgameEvalTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(MidgameEvalTest);
  CPPUNIT_TEST(testInit);
  CPPUNIT_TEST(testConsistentUpdate);
  CPPUNIT_TEST(testSymmetry);
  CPPUNIT_TEST_SUITE_END();
public:
  void testInit();
  void testConsistentUpdate();
  void testSymmetry();
};

CPPUNIT_TEST_SUITE_REGISTRATION(MidgameEvalTest);

using namespace osl;
using namespace osl::eval;
using namespace osl::eval::experimental;
extern bool isShortTest;

void MidgameEvalTest::testInit()
{
  CPPUNIT_ASSERT(MidgameEval::setUp());    
}

void MidgameEvalTest::testConsistentUpdate()
{
  consistencyTestUpdate<MidgameEval>();
}

void MidgameEvalTest::testSymmetry()
{
  using namespace osl;

  std::ifstream ifs(OslConfig::testCsaFile("FILES"));
  CPPUNIT_ASSERT(ifs);
  std::string file_name;
  for (int i=0;i<(isShortTest ? 10 : 200) && (ifs >> file_name) ; i++)
  {
    if (file_name == "") 
      break;
    file_name = OslConfig::testCsaFile(file_name);

    const Record rec=CsaFile(file_name).getRecord();
    const vector<osl::Move> moves=rec.getMoves();

    NumEffectState state(rec.getInitialState());
    MidgameEval eval(state);
    for (unsigned int i=0; i<moves.size(); i++) {
      const Move m = moves[i];
      ApplyMoveOfTurn::doMove(state, m);
      eval.update(state, m);

      NumEffectState state_r(state.rotate180());
      MidgameEval eval_r(state_r);
      if (abs(eval.value() - -eval_r.value()) > 2)
      {
	std::cerr << state << state_r << "\n";
	eval.showEvalSummary(state);
	std::cerr << "\n";
	eval.showEvalSummary(state_r);
      }
      CPPUNIT_ASSERT(abs(eval.value() - -eval_r.value()) <= 2);
#if 0
      // currentry broken by KingArea
      NumEffectState state_h(state.flipHorizontal());
      MidgameEval eval_h(state_h);
      if (eval.value() != eval_h.value())
      {
	std::cerr << state << state_h << "\n";
	eval.showEvalSummary(state);
	std::cerr << "\n";
	eval.showEvalSummary(state_h);
      }
      CPPUNIT_ASSERT_EQUAL(eval.value(), eval_h.value());
#endif
    }
  }
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
